from argparse import ArgumentParser
from typing import Literal, get_args

from httpx._models import Response
from rich.prompt import Prompt

from satoricli.api import client
from satoricli.cli.utils import autoformat, console

from .base import BaseCommand

ACTIONS = Literal["show", "create", "update", "delete"]


class TemplateCommand(BaseCommand):
    name = "template"

    def register_args(self, parser: ArgumentParser):
        parser.add_argument("id")
        parser.add_argument(
            "action",
            nargs="?",
            choices=get_args(ACTIONS),
            default="show",
        )

    def __call__(self, id: str, action: ACTIONS, **kwargs):
        if id == "create":
            action = "create"
        if action == "show":
            info = client.get(f"/templates/{id}").json()
            content = info.pop("content")
            autoformat(info)
            console.print("Content:\n", content)
        elif action == "update":
            console.print("Please use the web interface to update templates")
        elif action == "create":
            title = Prompt.ask("Enter the template title")
            description = Prompt.ask("Enter the template description")
            severity = Prompt.ask(
                "Select the template severity",
                choices=["INFO", "LOW", "MEDIUM", "HIGH", "CRITICAL", "BLOCKER"],
            )
            console.print(
                "Enter your text (press [b]Enter[/] on an empty line to finish):"
            )
            lines = []
            while True:
                line = console.input()
                if not line:
                    break
                lines.append(line)
            multiline_text = "\n".join(lines)
            res: Response = client.post(
                "/templates",
                json={
                    "title": title,
                    "description": description,
                    "severity": severity,
                    "content": multiline_text,
                },
            )
            if res.is_success:
                console.print("Template created")
            else:
                console.print(f"Error creating template: {res.text}")
        elif action == "delete":
            client.delete(f"/templates/{id}")



import shutil
import subprocess
import sys
from argparse import ArgumentParser

from ..utils import error_console
from .base import BaseCommand

PROMPT = (
    "Check if all the github repositories from satorici are downloaded: "
    "playbooks, playbook-validator, satori-cli, satori-docs. "
    "If they are not downloaded do it. "
    "Help Satori's user to build a playbook. "
)


class AiCommand(BaseCommand):
    name = "ai"

    def register_args(self, parser: ArgumentParser):
        parser.add_argument("prompt", metavar="PROMPT", help="Describe the playbook you want")

    def __call__(self, prompt: str, **kwargs):
        if not shutil.which("claude"):
            error_console.print("ERROR: claude CLI is not installed.")
            error_console.print("Install it: npm install -g @anthropic-ai/claude-code")
            return 1

        full_prompt = PROMPT + prompt
        result = subprocess.run(
            ["claude", "--dangerously-skip-permissions", "-p", full_prompt],
            check=False,
        )
        return result.returncode



from argparse import ArgumentParser

from satoricli.api import client
from satoricli.cli.utils import BootstrapTable, autotable

from .base import BaseCommand


class TemplatesCommand(BaseCommand):
    name = "templates"

    def register_args(self, parser: ArgumentParser):
        parser.add_argument("-p", "--page", type=int, default=1)
        parser.add_argument("-l", "--limit", type=int, default=20)

    def __call__(self, page: int, limit: int, **kwargs):
        info = client.get("/templates", params={"page": page, "limit": limit}).json()
        autotable(BootstrapTable(**info), page=page, limit=limit)



from argparse import ArgumentParser
import subprocess
import sys
import webbrowser

from satoricli.utils import load_config, save_config

from .base import BaseCommand
from ..utils import console, error_console


class InstallCommand(BaseCommand):
    name = "install"

    def register_args(self, parser: ArgumentParser):
        pass

    def __call__(self, **kwargs):
        # Open browser to get token
        url = "https://satori.ci/user-settings/#user-api-token"
        webbrowser.open(url)
        console.print(
            "Log in to Satori CI and get your token from the 'User API Token' page at: [b blue]{url}[/]"
        )

        # Get token from user
        token = console.input("[warning]Copy and paste your token here: [/]")
        token = token.strip()

        # Save token
        config = load_config()
        config.setdefault(kwargs["profile"], {})["token"] = token
        console.print(f"\nSatori CLI Token updated for profile {kwargs['profile']}")
        save_config(config)

        # Install v2
        command = [
            sys.executable,
            "-m",
            "pip",
            "install",
            "git+https://github.com/satorici/cli-v2",
        ]
        result = subprocess.run(command, capture_output=True, text=True)
        if result.returncode != 0: # error
            error_console.print("\n[bold red]Failed to install Satori CLI v2:[/]")
            error_console.print(result.stderr)
            return 1
        else:  # ok
            console.print("\nSatori CLI v2 installed successfully")



import fcntl
import os
import select
import signal
import struct
import sys
import termios
import time
import tty
from argparse import ArgumentParser
from typing import Mapping

import httpx
import paramiko
from paramiko.ssh_exception import NoValidConnectionsError
from rich.progress import Progress

from satoricli.api import client as v1_client

from .base import BaseCommand


def get_terminal_size():
    buf = fcntl.ioctl(sys.stdout, termios.TIOCGWINSZ, b"\x00" * 8)
    rows, cols = struct.unpack("hh", buf[:4])
    return cols, rows


def interactive_shell(host: str, token: str):
    ssh_client = paramiko.SSHClient()
    ssh_client.set_missing_host_key_policy(paramiko.AutoAddPolicy)

    tries = 0

    while tries < 3:
        try:
            ssh_client.connect(
                hostname=host,
                username="root",
                password=token,
                look_for_keys=False,
                allow_agent=False,
            )
            break
        except NoValidConnectionsError:
            time.sleep(1)
            tries += 1

    if tries >= 3:
        print("Can't connect to host")
        sys.exit(1)

    if transport := ssh_client.get_transport():
        transport.set_keepalive(30)

    cols, rows = get_terminal_size()
    channel = ssh_client.invoke_shell(term="xterm-256color", width=cols, height=rows)

    def resize_handler(signum, frame):
        cols, rows = get_terminal_size()
        channel.resize_pty(width=cols, height=rows)

    old_handler = signal.signal(signal.SIGWINCH, resize_handler)
    old_tty = termios.tcgetattr(sys.stdin)

    try:
        tty.setraw(sys.stdin.fileno())
        channel.settimeout(0.0)

        while True:
            r, _, _ = select.select([channel, sys.stdin], [], [])

            if channel in r:
                try:
                    data = channel.recv(1024)

                    if not data:
                        break

                    sys.stdout.buffer.write(data)
                    sys.stdout.buffer.flush()
                except Exception:
                    break

            if sys.stdin in r:
                data = os.read(sys.stdin.fileno(), 1024)
                channel.send(data)
    finally:
        termios.tcsetattr(sys.stdin, termios.TCSADRAIN, old_tty)
        signal.signal(signal.SIGWINCH, old_handler)
        channel.close()
        ssh_client.close()


def remove_none_values(d: Mapping):
    return {k: v for k, v in d.items() if v is not None}


class ShellCommand(BaseCommand):
    name = "shell"

    def register_args(self, parser: ArgumentParser):
        parser.add_argument("--image")
        parser.add_argument("--memory", type=int)
        parser.add_argument("--cpu", type=int)
        parser.add_argument("--region", action="append", default=[])
        parser.add_argument("--timeout", type=int)

    def __call__(self, cpu, memory, image, region, timeout, **kwargs):
        client = httpx.Client(
            base_url="https://api-v2.satori.ci",
            headers=v1_client.headers,
        )

        container_settings = remove_none_values(
            {"cpu": cpu, "memory": memory, "image": image}
        )

        res = client.post(
            "/ssh_sessions",
            json={
                "regions": list(region),
                "container_settings": container_settings,
                "timeout": timeout,
            },
        )
        res.raise_for_status()
        id = res.json()["id"]

        with Progress(transient=True) as progress:
            progress.add_task("Waiting for host", total=None)

            while True:
                try:
                    res = client.get(f"/ssh_sessions/{id}")
                    res.raise_for_status()
                    session_data = res.json()
                    break
                except Exception:
                    time.sleep(2)

        interactive_shell(session_data["host"], session_data["token"])
